/*
Copyright © 2023 AGECEM & Victor Lacasse-Beaudoin
*/
package cmd

import (
	"fmt"
	"log"

	"git.agecem.com/agecem/bottin-ag/config"
	"git.agecem.com/agecem/bottin-ag/webcontent"
	"git.agecem.com/agecem/bottin-ag/webhandler"
	"github.com/labstack/echo/v4"
	"github.com/labstack/echo/v4/middleware"
	"github.com/spf13/cobra"
)

// webCmd represents the web command
var webCmd = &cobra.Command{
	Use:   "web",
	Short: "Start the webserver",
	Run: func(cmd *cobra.Command, args []string) {
		cfg, err := config.UnmarshalConfig()
		if err != nil {
			log.Fatal(err)
		}

		e := echo.New()

		e.Renderer = webcontent.TemplateHTMLFS()

		e.Pre(middleware.AddTrailingSlash())

		handler := webhandler.New()

		webhandler.DeclareRoutes(e, &handler)

		e.Start(fmt.Sprintf(":%d", cfg.Web.Port))
	},
}

func init() {
	rootCmd.AddCommand(webCmd)
}
